import React, { useState } from 'react';
import { Input, Button, Form, Modal, ModalProps, message } from 'antd';
import styles from './login.less';
import { history, useModel } from 'umi';
import { MobileFilled, LockFilled } from '@ant-design/icons';
import { useCallback } from 'react';
import { useEffect } from 'react';
import { login } from '@/services/user';
import {
  firstCharToUpperCase,
  handleRequest,
  isClient,
  passwordReg,
} from '@/utils/tool';
import { useForm } from 'antd/lib/form/Form';
import classNames from 'classnames';
import SmsInputer from './SmsInputer';
import { memoize } from 'lodash';
import { Rule } from 'antd/lib/form';
import storage from '@/utils/storage';
import { useRef } from 'react';
import { useLayoutEffect } from 'react';
import { fetchApi } from '@/utils/request';

export default function Login() {
  const [errText, setErrText] = useState('');
  const [account, setAccount] = useState('');
  const [password, setPassword] = useState('');
  const [loading, setLoading] = useState(false);
  const [regModalVisible, setRegModalVisible] = useState(false);
  const { refresh } = useModel('@@initialState');
  const buttonRef = useRef<HTMLElement>(null);
  // const { loading, signin } = useModel('useAuthModel');

  const onLogin = useCallback(async () => {
    if (!account) {
      setErrText('账号/手机号不能为空');
      return;
    }
    if (!password) {
      setErrText('密码不能为空');
      return;
    }
    setLoading(true);
    const res = await login(account, password);
    setLoading(false);
    handleRequest(res)
      .error(() => {
        setErrText(res.message!);
      })
      .success(() => {
        // 记录账号密码
        if (isClient) {
          storage.set('account', account, true);
          storage.set('password', password, true);
        }
        // history.push('/');
        // window.location.href = '/';
        history.replace('/');
        refresh();
        // window.location.reload();
      });
  }, [account, password]);

  useEffect(() => {
    setErrText('');
  }, [account, password]);

  useLayoutEffect(() => {
    if (isClient) {
      const iAccount = storage.get('account');
      const iPass = storage.get('password');
      if (iAccount && iPass) {
        setAccount(iAccount);
        setPassword(iPass);
        setTimeout(() => {
          buttonRef.current?.click();
        }, 500);
      }
    }
  }, [buttonRef]);

  return (
    <div className={styles.login}>
      <iframe
        src="bgAnimate/bganimat.html"
        frameBorder="0"
        className={styles.loginBg}
        // class="login_bg"
      ></iframe>
      <div className={styles.title}>
        <div>欢迎使用</div>
        <div>locking</div>
      </div>
      <Form className={styles.form}>
        <Input
          size="large"
          prefix={<MobileFilled className={styles.icon} />}
          disabled={loading}
          className={styles.mb24}
          value={account}
          onChange={(e) => setAccount(e.target.value)}
        />
        <Input
          size="large"
          prefix={<LockFilled className={styles.icon} />}
          type="password"
          disabled={loading}
          className={styles.mb4}
          value={password}
          onChange={(e) => setPassword(e.target.value)}
        />
        <div className={styles.errText}>{errText}</div>
        <div className={styles.btnGroup}>
          <Button
            ref={buttonRef}
            loading={loading}
            type="primary"
            className={styles.btn}
            onClick={onLogin}
            htmlType="submit"
          >
            登录
          </Button>
          <Button
            className={styles.btn}
            onClick={() => setRegModalVisible(true)}
          >
            注册
          </Button>
        </div>
      </Form>
      <RegisterModal
        visible={regModalVisible}
        onCancel={() => setRegModalVisible(false)}
        onFninsh={(nextAccount, nextPassword) => {
          setAccount(nextAccount);
          setPassword(nextPassword);
          setRegModalVisible(false);
        }}
      />
    </div>
  );
}

interface RegisterModalProps extends ModalProps {
  onFninsh: (phone: string, password: string) => void;
}

const defaultValidateMessages = {
  required: '${label}不能为空',
};

const requiredRules: (...arg: Rule[]) => Rule[] = memoize((...otherRules) => [
  { required: true },
  ...otherRules,
]);

function RegisterModal(props: RegisterModalProps) {
  const { className, onFninsh, ...restProps } = props;
  const [form] = useForm();
  useEffect(() => {
    if (props.visible) {
      form.resetFields();
    }
  }, [props.visible, form]);

  const onSubmit = useCallback(async () => {
    try {
      const fields = await form.validateFields();
      const params = firstCharToUpperCase(fields);
      const res = await fetchApi('company/registerCompany', params);
      handleRequest(res).success(() => {
        message.success('注册成功');
        onFninsh(fields.phone, fields.password);
      });
    } catch (e) {}
  }, [form]);

  return (
    <Modal
      {...restProps}
      title="注册企业账户"
      width={720}
      footer={null}
      className={classNames(styles.modal, className)}
      maskClosable={false}
    >
      <Form
        form={form}
        colon
        className={classNames(styles.form)}
        labelCol={{ span: 6 }}
        validateMessages={defaultValidateMessages}
        requiredMark={false}
      >
        <Form.Item name="compAllName" label="公司名称" rules={requiredRules()}>
          <Input />
        </Form.Item>

        <Form.Item name="userName" label="管理员" rules={requiredRules()}>
          <Input />
        </Form.Item>

        <Form.Item name="phone" label="登录手机号" rules={requiredRules()}>
          <Input />
        </Form.Item>

        <Form.Item
          name="password"
          label="密码"
          rules={requiredRules({
            message: '密码8~16位,包含大写字母、小写字母和数字',
            pattern: passwordReg,
          })}
        >
          <Input type="password" />
        </Form.Item>

        <Form.Item
          name="confirmPassword"
          label="确认密码"
          dependencies={['password']}
          rules={requiredRules(({ getFieldValue }) => ({
            validator(_, value) {
              if (!value || getFieldValue('password') === value) {
                return Promise.resolve();
              }
              return Promise.reject(new Error('确认密码和密码不一致'));
            },
          }))}
        >
          <Input type="password" />
        </Form.Item>

        <Form.Item noStyle dependencies={['phone']}>
          {() => (
            <Form.Item
              name="verificationCode"
              label="验证码"
              rules={requiredRules()}
            >
              <SmsInputer phone={form.getFieldValue('phone')} />
            </Form.Item>
          )}
        </Form.Item>

        <Form.Item label=" " className={styles.btnGroup} colon={false}>
          <Button className={styles.btn} type="primary" onClick={onSubmit}>
            确定
          </Button>
          <Button className={styles.btn} onClick={props.onCancel}>
            取消
          </Button>
        </Form.Item>
      </Form>
    </Modal>
  );
}
