import { fileMng, msgNotifySubject } from '@/utils/hooks';
import { firstCharToLowerCase } from '@/utils/tool';

const gatewayPort = window.systemConfig?.gatewayPort || 7888;
const taskSyncSkUrl = `ws://127.0.0.1:${gatewayPort}/websocket/subscriptionTaskSync`;
const msgNotifySkUrl = `ws://127.0.0.1:${gatewayPort}/websocket/subscriptionMessageNotify`;

enum SocketState {
  CONNECTING = 0, // 连接尚未建立
  OPEN, // WebSocket的链接已经建立
  CLOSING, // 连接正在关闭
  CLOSED, // 连接已经关闭或不可用
}

let taskSyncSocket: WebSocket;
let msgNotifySocket: WebSocket;
function initTaskSyncSocket() {
  taskSyncSocket = new WebSocket(taskSyncSkUrl);
  taskSyncSocket.onopen = () => {
    console.log('taskSyncSocket connection');
  };
  taskSyncSocket.onmessage = ({ data }) => {
    fileMng.receiveMessage(data);
  };
  taskSyncSocket.onerror = (...args) => {
    console.log('taskSyncSocket error:', args);
  };
  taskSyncSocket.onclose = (...args) => {
    console.log('taskSyncSocket close:', args);
    setTimeout(() => initTaskSyncSocket(), 5000);
  };
}
function initMessageNotifySocket() {
  msgNotifySocket = new WebSocket(msgNotifySkUrl);
  msgNotifySocket.onopen = () => {
    console.log('taskSyncSocket connection');
  };
  msgNotifySocket.onmessage = ({ data }) => {
    // fileMng.receiveMessage(data);
    try {
      const originNotifyMessage = firstCharToLowerCase(JSON.parse(data)) as any;
      msgNotifySubject.notifyObservers(originNotifyMessage);
    } catch (e) {
      console.error('socket notify message 解析失败:', e, data);
    }
  };
  msgNotifySocket.onerror = (...args) => {
    console.log('taskSyncSocket error:', args);
  };
  msgNotifySocket.onclose = (...args) => {
    console.log('taskSyncSocket close:', args);
    setTimeout(() => initTaskSyncSocket(), 5000);
  };
}
export function sendSocketMessage(message: string) {
  if (taskSyncSocket && taskSyncSocket.readyState === SocketState.OPEN) {
    taskSyncSocket.send(message);
  }
}
export function initialWebsocket() {
  initTaskSyncSocket();
  initMessageNotifySocket();
}

// export function use
