import { fetchLocalApi } from '@/utils/request';
import { isClient } from '@/utils/tool';
import { DATA } from './API';
import { sendSocketMessage } from './socket';

const noop = () => {};
const safeCall = <T>(f: T, repalceF = noop) => (isClient ? f : repalceF);

const ipcRenderer = window.ipcRenderer;

const system = {
  closeWindow() {
    ipcRenderer.invoke('window:close');
  },
  zoomWindow() {
    ipcRenderer.invoke('window:zoom');
  },
  minimizeWindow() {
    ipcRenderer.invoke('window:minimize');
  },
  openUrl(url: string) {
    if (!isClient) {
      window.open(url, 'blank');
      return;
    }
    ipcRenderer.invoke('open-browser', url);
  },
  // 选择文件夹
  chooseFolder: safeCall(async () => {
    const res = await ipcRenderer.invoke('select-folder');
    const { canceled, filePaths } = res;
    if (canceled) return null;
    return filePaths[0] as string;
  }),
  openFilePosition: safeCall((message: DATA.SocketFileMsg) => {
    ipcRenderer.invoke('open-file-position', message);
  }),
  storage: {
    set: safeCall((key: string, value: any) => {
      ipcRenderer.invoke('storage:set', { key, value });
    }),
    remove: safeCall((key: string) => {
      ipcRenderer.invoke('storage:remove', { key });
    }),
  },
  /* 发往网关的请求 */
  // 登陆
  login: async (userId: string, userPhone: string, api: string) => {
    const res = await fetchLocalApi<null>(
      'login',
      { userId, userPhone, api },
      { silent: true },
    );
    sendSocketMessage(`login:${userPhone}`);
    return res;
  },
  // 退出登陆
  logout: async () => {
    return await fetchLocalApi<null>('logout');
  },
  // 发送下载同步文件夹的指令
  syncProjects: (projectIds: string[]) => {
    return fetchLocalApi<null>('syncFolderToWorkSpace', {
      ids: projectIds.join(','),
    });
  },
};

if (isClient) {
  window.addIpcRendererListener(
    'request-resync-file',
    (e, msg: DATA.SocketFileMsg) => {
      fetchLocalApi('restartTask', { taskIds: msg.taskId });
    },
  );
  window.addIpcRendererListener(
    'request-download-file',
    (e, msg: DATA.SocketFileMsg) => {
      fetchLocalApi('restartTask', { taskIds: msg.taskId });
    },
  );
}

export default system;
